@@ -7,11 +7,12 @@ from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 from trytond.tools import grouped_slice
 
-__all__ = ['PurchaseRequest', 'Purchase']
-__metaclass__ = PoolMeta
+__all__ = ['PurchaseRequest', 'Purchase',
+    'HandlePurchaseCancellationException']
 
 
 class PurchaseRequest:
+    __metaclass__ = PoolMeta
     __name__ = 'purchase.request'
 
     @classmethod
@@ -48,6 +49,7 @@ class PurchaseRequest:
 
 
 class Purchase:
+    __metaclass__ = PoolMeta
     __name__ = 'purchase.purchase'
 
     @classmethod
@@ -81,3 +83,25 @@ class Purchase:
     def cancel(cls, purchases):
         super(Purchase, cls).cancel(purchases)
         cls._sale_supply_process(purchases)
+
+
+class HandlePurchaseCancellationException:
+    __metaclass__ = PoolMeta
+    __name__ = 'purchase.request.handle.purchase.cancellation'
+
+    def transition_cancel_request(self):
+        pool = Pool()
+        SaleLine = pool.get('sale.line')
+        Move = pool.get('stock.move')
+
+        next_state = super(HandlePurchaseCancellationException,
+            self).transition_cancel_request()
+        moves = []
+        for sub_ids in grouped_slice(Transaction().context['active_ids']):
+            sale_lines = SaleLine.search([
+                    ('purchase_request', 'in', sub_ids),
+                    ])
+            moves += [m for line in sale_lines for m in line.moves]
+        if moves:
+            Move.cancel(moves)
+        return next_state
